#ifndef SPODS_PhysicsObjectBuilder
#define SPODS_PhysicsObjectBuilder

// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include "TClonesArray.h"
#include "TParticle.h"
#include "TRandom3.h"
#include "fastjet/ClusterSequence.hh"

#include "YamlConfig.h"
#include "TDatabasePDG.h"

namespace SPODS 
{
class PhysicsObjectBuilder
{
protected:
   TClonesArray* fGenParticles;
   YamlConfig* fConfig;
   fastjet::JetDefinition* jet_def;
   TRandom3 tRandom;
   TDatabasePDG* databasePDG;
   int fNGenParticles;
   void GetJets(TClonesArray* genParticles, TClonesArray* jets);
   void GetNConstituents(const fastjet::PseudoJet& jet, 
	int& N, int& nCharged, int& nHadron) ;//const;
public:
   PhysicsObjectBuilder(SPODS::YamlConfig* config);
   void LoadGenParticles(TClonesArray* genParticles);
   void MakeElectrons(TClonesArray *electrons);
   void MakeMuons(TClonesArray *muons);
   void MakePhotons(TClonesArray *photons);
   void MakeJets(TClonesArray *jets);
   void MakeMet(TClonesArray* physicsObjects, 
	double& met, double& phi, double& sumPt,
	double& trueMet, double& truePhi, double& trueSumPt
  	);
   void MakeIso(TClonesArray* physicsObjects, std::string type, 
	  float coneSize, float minPt, std::string particle); // particle: "electron", "photon", "muon"
   void BTagJets(TClonesArray *jets, TClonesArray* allGenParticles);
   void CombinePhysicsObjects(TClonesArray* electrons, TClonesArray* muons, 
	TClonesArray* photons, TClonesArray* jets, TClonesArray* combined);
   void CombinePhysicsObjects(TClonesArray* objects1, TClonesArray* objects2, 
	TClonesArray* combined);
   int GetChargeFromPDG(const int pdg) const;
};

}
#endif


