#ifndef SPODS_YamlConfig
#define SPODS_YamlConfig

// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <string>
#include <vector>
#include <utility>
#include <iostream>

namespace SPODS 
{
class YamlConfig
{
 public:
   YamlConfig(std::string configFileName);
   void Print(std::ostream& = std::cout) const;
   void PrintVector(const std::vector<float>) const;
   void PrintVectorOfPairs(const std::vector<std::pair<float,float> >) const;

   int maxEvents;
   int maxGenParticles;
   int maxElectrons;
   int maxMuons;
   int maxPhotons;
   int maxJets;

   float etaElectronMax;
   float etaMuonMax;
   float etaPhotonMax;
   float etaJetMax;
   float etaMetMax;

   float ptElectronMin;
   float ptMuonMin;
   float ptPhotonMin;
   float ptJetMin;

   std::string jetAlg;
   float jetAlgR;
   int jetVetoNPartMin;
   int jetVetoNChargMin;
   int jetVetoNHadMin;

   std::string muonIsoType;
   float muonIsoConeSize;
   float muonIsoMinPt;
   std::string electronIsoType;
   float electronIsoConeSize;
   float electronIsoMinPt;
   std::string photonIsoType;
   float photonIsoConeSize;
   float photonIsoMinPt;

   float bTagEff;
   float bTagMistagRate;

   //Detector Resolutions
   std::vector<std::pair<float,float> > muonResEtaRanges;
   std::vector<float> muonResConstTerm;
   std::vector<float> muonResSqrtTerm;
   std::vector<float> muonResHighPtCut;
   std::vector<float> muonResHighPtConstTerm;
   std::vector<float> muonResHighPtSqrtTerm;

   std::vector<std::pair<float,float> > photonResEtaRanges;
   std::vector<float> photonResConstTerm;
   std::vector<float> photonResOneOverETerm;
   std::vector<float> photonResOneOverSqrtETerm;

   float electronResPtCut;
   float electronResConstantTerm;

   std::vector<float> jetResPtCut;
   std::vector<std::pair<float,float> > jetResEtaRanges;
   std::vector<float> jetResConstTerm;
   std::vector<float> jetResOneOverETerm;
   std::vector<float> jetResOneOverSqrtETerm;
};

}
#endif


