
// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include "HepMCImporter.h"
#include "TParticle.h"
#include "TObjArray.h"
#include "TClonesArray.h"
#include "HepMC/GenEvent.h"
#include "HepMC/GenParticle.h"
#include <iostream>


namespace SPODS
{
HepMCImporter::HepMCImporter(const std::string inFileName):
	fIO_GenEvent(inFileName.c_str(),std::ios::in),
	fCurrentGenEvent(),
	fNumberOfParticles(0),
	fDoneReading(false)
{
	//fIO_GenEvent = HepMC::IO_GenEvent(inFileName.c_str(),std::ios::in);
	//fCurrentGenEvent = HepMC::GenEvent();
	//fNumberOfParticles = 0;
}

void
HepMCImporter::GenerateEvent()
{
	fCurrentGenEvent.clear();
    int rdState = fIO_GenEvent.rdstate();
    if(rdState == 0)
    {
	  fDoneReading =  fIO_GenEvent.fill_next_event(&fCurrentGenEvent);
	  fNumberOfParticles = fCurrentGenEvent.particles_size();
    }
    else
    {
      std::cerr << "Error: HepMCImporter:GenerateEvent(): fIO_GenEvent.rdstate() = " << rdState << std::endl;
      fDoneReading = true;
    }
}

int 
HepMCImporter::ImportParticles(TClonesArray *particles, const char* option)
{
    int nparts = 0;
    TClonesArray &clonesParticles = *particles;
    clonesParticles.Clear();
	
    HepMC::GenEvent::particle_const_iterator hepParticle = 
					fCurrentGenEvent.particles_begin();
    HepMC::GenEvent::particle_const_iterator hepParticleEnd = 
					fCurrentGenEvent.particles_end();
    if (!strcmp(option,"All"))
    {
	for (;hepParticle != hepParticleEnd; hepParticle++)
	{
	    //(*hepParticle)->print();
	    float vx=-999;
	    float vy=-999;
	    float vz=-999;
	    float vt=-999;
	    if((*hepParticle)->production_vertex())
	    {
	        //(*hepParticle)->production_vertex()->print();
                vx = (*hepParticle)->production_vertex()->position().x();  // [mm]
                vy = (*hepParticle)->production_vertex()->position().y();  // [mm]
                vz = (*hepParticle)->production_vertex()->position().z();  // [mm]
                vt = (*hepParticle)->production_vertex()->position().t(); // [mm/c] 
	    }
            new(clonesParticles[nparts]) TParticle(
                (*hepParticle)->pdg_id(),
                (*hepParticle)->status(),
                -1,//(*hepParticle)->mother1() - 1,
                -1,//(*hepParticle)->mother2() - 1,
                -1,//(*hepParticle)->daughter1() - 1,
                -1,//(*hepParticle)->daughter2() - 1,
                (*hepParticle)->momentum().px(),     // [GeV/c]
                (*hepParticle)->momentum().py(),     // [GeV/c]
                (*hepParticle)->momentum().pz(),     // [GeV/c]
                (*hepParticle)->momentum().e(),      // [GeV]
		vx,
		vy,
		vz,
		vt);
	    nparts++;
	    if (nparts >= particles->GetSize()) break;
	}
    }
    else
    {
	for (;hepParticle != hepParticleEnd; hepParticle++)
	{
	  if ((*hepParticle)->status() == 1)
	  {
	    //(*hepParticle)->production_vertex()->print();
	    //(*hepParticle)->print();
            new(clonesParticles[nparts]) TParticle(
                (*hepParticle)->pdg_id(),
                (*hepParticle)->status(),
                -1,//(*hepParticle)->mother1() - 1,
                -1,//(*hepParticle)->mother2() - 1,
                -1,//(*hepParticle)->daughter1() - 1,
                -1,//(*hepParticle)->daughter2() - 1,
                (*hepParticle)->momentum().px(),     // [GeV/c]
                (*hepParticle)->momentum().py(),     // [GeV/c]
                (*hepParticle)->momentum().pz(),     // [GeV/c]
                (*hepParticle)->momentum().e(),      // [GeV]
                (*hepParticle)->production_vertex()->position().x(),  // [mm]
                (*hepParticle)->production_vertex()->position().y(),  // [mm]
                (*hepParticle)->production_vertex()->position().z(),  // [mm]
                (*hepParticle)->production_vertex()->position().t()); // [mm/c] 
	    nparts++;
	    if (nparts >= particles->GetSize()) break;
	    }
	}
    }
    return nparts;
}

int
HepMCImporter::GetNMPI() const
{
  return fCurrentGenEvent.mpi();
}

double
HepMCImporter::GetWeight() const
{
  const HepMC::WeightContainer& weights = fCurrentGenEvent.weights();
  double result = 1.0;
  if (!weights.empty())
  {
    result = weights.front();
  }
  return result;
}

}
