
// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include "POUtilities.h"
#include "PhysicsObjectFunctions.h"

#include <vector>
#include <algorithm>

bool ptSortFn(TParticle i,TParticle j) {return i.Pt() > j.Pt();}

void removeLowPt(TClonesArray* inParticles, float minPt) //of TParticles or TLorentzVectors
{
	std::vector<TParticle> tmpVec;
	unsigned nPart = inParticles->GetEntries();
	for(unsigned iPart = 0; iPart < nPart; iPart++)
	{
	  TParticle* particle = (TParticle*) (inParticles->At(iPart));
	  float pt = particle->Pt();
	  if (pt >= minPt)
	  {
		tmpVec.push_back(*particle);
	  }
	}

	inParticles->Clear();

	//Now to sort by Pt
	std::sort(tmpVec.begin(),tmpVec.end(),ptSortFn);

	unsigned iOut = 0;
	unsigned nGood = tmpVec.size();
	for(unsigned iGood=0; iGood<nGood; iGood++)
	{
		new(inParticles->operator[](iOut)) TParticle(tmpVec[iGood]);
		iOut++;
	}
};

void removeIsoLepPhFromJets(TClonesArray* jets, TClonesArray* muons, TClonesArray* electrons, TClonesArray* photons)
{
	std::vector<TParticle> tmpVec;
	unsigned nJets = jets->GetEntries();
	unsigned nMuons = muons->GetEntries();
	unsigned nElectrons = electrons->GetEntries();
	unsigned nPhotons = photons->GetEntries();
	for(unsigned iJet = 0; iJet < nJets; iJet++)
	{
      bool cleanJet = true;
	  TParticle* jet = (TParticle*) (jets->At(iJet));
      TLorentzVector jetP;
      jet->Momentum(jetP);
	  for(unsigned iPart = 0; iPart < nMuons; iPart++)
      {
	    TParticle* part = (TParticle*) (muons->At(iJet));
	    float pt = part->Pt();
        float iso = getIso(part);
        if(iso/pt < 0.5)
        {
            TLorentzVector partP;
            part->Momentum(partP);
            if(jetP.DeltaR(partP)<0.3)
            {
                cleanJet = false;
                break;
            }
        }
      }//Muon Loop
      if (!cleanJet)
        continue;

	  for(unsigned iPart = 0; iPart < nElectrons; iPart++)
      {
	    TParticle* part = (TParticle*) (electrons->At(iJet));
	    float pt = part->Pt();
        float iso = getIso(part);
        if(iso/pt < 0.5)
        {
            TLorentzVector partP;
            part->Momentum(partP);
            if(jetP.DeltaR(partP)<0.3)
            {
                cleanJet = false;
                break;
            }
        }
      }//Electron Loop
      if (!cleanJet)
        continue;

	  for(unsigned iPart = 0; iPart < nPhotons; iPart++)
      {
	    TParticle* part = (TParticle*) (photons->At(iJet));
	    float pt = part->Pt();
        float iso = getIso(part);
        if(iso/pt < 0.5)
        {
            TLorentzVector partP;
            part->Momentum(partP);
            if(jetP.DeltaR(partP)<0.3)
            {
                cleanJet = false;
                break;
            }
        }
      }//Photon Loop
      if (!cleanJet)
        continue;

      tmpVec.push_back(*jet);
	}//Jet Loop

	jets->Clear();

	unsigned iOut = 0;
	unsigned nGood = tmpVec.size();
	for(unsigned iGood=0; iGood<nGood; iGood++)
	{
		new(jets->operator[](iOut)) TParticle(tmpVec[iGood]);
		iOut++;
	}

}
