#!/usr/bin/python

import random
from math import *

class MakeHepMC:
  def __init__(self):
    self.nMu = 1
    self.nEl = 1
    self.nPh = 1
    self.nHad = 2
    self.nJet = 1
    self.bJets = False
    self.muPtCuts = [20.0,200.0]
    self.elPtCuts = [20.0,200.0]
    self.phPtCuts = [30.0,200.0]
    self.jetPtCuts = [15.0,200.0]
    self.weight = None
    self.mpi = 0

    self.muEtaMax = 2.4
    self.elEtaMax = 2.4
    self.phEtaMax = 2.4
    self.jetEtaMax = 3.0 

  def getRandom4Vec(self,minPt,maxPt,maxEta):
    eta = random.random()*maxEta
    if random.random()>0.5:
        eta = -eta
    theta = 2*atan(exp(-eta))
    pt = random.random()*(maxPt-minPt)+minPt
    phi = random.random()*2*pi
    x = pt*cos(phi)
    y = pt*sin(phi)
    z = pt/tan(theta)
    m = 0.0
    E = sqrt(x**2+y**2+z**2+m**2)
    return (x,y,z,E)

  def generate(self,nEvts):
    self.lines = []
    self.lines.append("HepMC::Version SPODS Particle Gun\n")
    self.lines.append("HepMC::IO_GenEvent-START_EVENT_LISTING\n")

    nBJets = 0
    if self.bJets:
        nBJets = self.nJet

    nParticles = self.nMu+self.nEl+self.nPh+self.nHad*self.nJet+nBJets

    for evt in range(nEvts):
      if self.weight == None:
        self.lines.append("E %i %i 200.0 0.14 0.014 1 1 1 0 1 0 0\n" % (evt,self.mpi))
      else:
        self.lines.append("E %i %i 200.0 0.14 0.014 1 1 1 0 1 0 2 %f 999.24\n" % (evt,self.mpi, self.weight))
      self.lines.append("U GEV MM\n")
      self.lines.append("V 0 0 0.0 0.0 0.0 0.0 0 %i 0\n" % nParticles)
      
      for iMu in range(self.nMu):
        tmp = "P 0 13 %.5f %.5f %.5f %.5f 0.105 1 0.0 0.0 -1 0\n" % self.getRandom4Vec(self.muPtCuts[0],self.muPtCuts[1],self.muEtaMax)
        self.lines.append(tmp)
      for iEl in range(self.nEl):
        tmp = "P 0 11 %.5f %.5f %.5f %.5f 0.0 1 0.0 0.0 -1 0\n" % self.getRandom4Vec(self.elPtCuts[0],self.elPtCuts[1],self.elEtaMax)
        self.lines.append(tmp)
      for iPh in range(self.nPh):
        tmp = "P 0 22 %.5f %.5f %.5f %.5f 0.0 1 0.0 0.0 -1 0\n" % self.getRandom4Vec(self.phPtCuts[0],self.phPtCuts[1],self.phEtaMax)
        self.lines.append(tmp)
      for iJet in range(self.nJet):
        if self.nHad<1:
            break
        jetVec = self.getRandom4Vec(self.jetPtCuts[0]/self.nHad,self.jetPtCuts[1]/self.nHad,self.jetEtaMax)
        for iHad in range(self.nHad):
          tmp = "P 0 211 %.5f %.5f %.5f %.5f 0.135 1 0.0 0.0 -1 0\n" % jetVec
          self.lines.append(tmp)
        if self.bJets:
          bNum = random.choice([5,511,521,10521,545,20543,553,555,5122,5312,5444])
          tmp = "P 0 %i %.5f %.5f %.5f %.5f 0.135 1 0.0 0.0 -1 0\n" % (bNum,jetVec[0],jetVec[1],jetVec[2],jetVec[3])
          self.lines.append(tmp)

    self.lines.append("HepMC::IO_GenEvent-END_EVENT_LISTING\n")

  def generateAntiIsolated(self,nEvts):
    self.lines = []
    self.lines.append("HepMC::Version SPODS Particle Gun\n")
    self.lines.append("HepMC::IO_GenEvent-START_EVENT_LISTING\n")

    nParticles = (self.nMu+self.nEl+self.nPh)*(self.nHad+1)

    for evt in range(nEvts):
      self.lines.append("E %i 0 200.0 0.14 0.014 1 1 1 0 1 0 0\n" % evt)
      self.lines.append("U GEV MM\n")
      self.lines.append("V 0 0 0.0 0.0 0.0 0.0 0 %i 0\n" % nParticles)
      
      for iMu in range(self.nMu):
        vec = self.getRandom4Vec(self.muPtCuts[0]/self.nHad,self.muPtCuts[1]/self.nHad,self.muEtaMax)
        tmp = "P 0 13 %.5f %.5f %.5f %.5f 0.105 1 0.0 0.0 -1 0\n" % vec
        self.lines.append(tmp)
        for iHad in range(self.nHad):
          tmp = "P 0 211 %.5f %.5f %.5f %.5f 0.135 1 0.0 0.0 -1 0\n" % vec
          self.lines.append(tmp)
      for iEl in range(self.nEl):
        vec = self.getRandom4Vec(self.elPtCuts[0]/self.nHad,self.elPtCuts[1]/self.nHad,self.elEtaMax)
        tmp = "P 0 11 %.5f %.5f %.5f %.5f 0.0 1 0.0 0.0 -1 0\n" % vec
        self.lines.append(tmp)
        for iHad in range(self.nHad):
          tmp = "P 0 211 %.5f %.5f %.5f %.5f 0.135 1 0.0 0.0 -1 0\n" % vec
          self.lines.append(tmp)
      for iPh in range(self.nPh):
        vec = self.getRandom4Vec(self.phPtCuts[0]/self.nHad,self.phPtCuts[1]/self.nHad,self.phEtaMax)
        tmp = "P 0 22 %.5f %.5f %.5f %.5f 0.0 1 0.0 0.0 -1 0\n" % vec
        self.lines.append(tmp)
        for iHad in range(self.nHad):
          tmp = "P 0 211 %.5f %.5f %.5f %.5f 0.135 1 0.0 0.0 -1 0\n" % vec
          self.lines.append(tmp)
    self.lines.append("HepMC::IO_GenEvent-END_EVENT_LISTING\n")

  def writeFile(self,name):
    f = open(name,"w")
    f.writelines(self.lines)
    f.close()


if __name__ == "__main__":
  import sys
  if len(sys.argv) < 2:
    print("Usage: hepmcParticleGun.py <outfilename.hepmc>")
  else:
    outfilename = sys.argv[1]
    hepmc = MakeHepMC()
    hepmc.generate(10)
    hepmc.writeFile(outfilename)
