// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <TSystem.h>
#include <TROOT.h>
#include <TClonesArray.h>
#include <TParticle.h>
#include <iostream>
#include <TFile.h>
#include <TTree.h>

#include "HepMCImporter.h"
#include "PhysicsObjectBuilder.h"

using namespace SPODS;

int main()
{

  TFile* f = new TFile("tree.root","RECREATE");
  TTree * tree = new TTree("tree","tree");

  TClonesArray * particles = new TClonesArray("TParticle",1000);
  TClonesArray * electrons = new TClonesArray("TParticle",10);
  TClonesArray * muons = new TClonesArray("TParticle",10);
  TClonesArray * photons = new TClonesArray("TParticle",10);
  TClonesArray * jets = new TClonesArray("TParticle",10);
  double met=0;
  double metPhi=0;
  tree->Branch("genParticles",particles);
  tree->Branch("electrons",electrons);
  tree->Branch("muons",muons);
  tree->Branch("photons",photons);
  tree->Branch("jets",jets);
  tree->Branch("met",&met,"met/F");
  tree->Branch("metPhi",&metPhi,"metPhi/F");

  HepMCImporter myImporter("test.hepmc");
  PhysicsObjectBuilder myBuilder;
  bool moreEvents=true;
  while(moreEvents)
  {
	myImporter.GenerateEvent();
	int nParts = myImporter.ImportParticles(particles);
	moreEvents = myImporter.Done();

	myBuilder.LoadGenParticles(particles);
	myBuilder.MakeElectrons(electrons);
	myBuilder.MakeMuons(muons);
	myBuilder.MakePhotons(photons);
	myBuilder.MakeJets(jets);
	myBuilder.MakeMet(met,metPhi);

	tree->Fill();
  }

  f->cd();
  tree->Write();

  return 0;
}
