// SPODS--Simple Physics Object based Detector Simulation
// Copyright (C) 2011, 2012, 2013 Justin Hugon

#include <TSystem.h>
#include <TROOT.h>
#include <TClonesArray.h>
#include <TParticle.h>
#include <iostream>
#include <TFile.h>
#include <TTree.h>

#include "HepMCImporter.h"
#include "PhysicsObjectBuilder.h"


#include <fstream>
#include <yaml-cpp/yaml.h>
#include <iostream>
#include "YamlReader.h"
#include "YamlConfig.h"

using namespace SPODS;

int main()
{
  SPODS::YamlReader yReaderSilly("silly.yaml");
  std::cout << "nEventsMax is " << yReaderSilly.GetValue("nEventsMax") << std::endl;

  SPODS::YamlConfig* config = new SPODS::YamlConfig("cmsConfig.yaml");
  std::cout << "maxPhotons is " << config->maxPhotons << std::endl;
  std::cout << "etaMuonMax is " << config->etaMuonMax << std::endl;
  std::cout << "ptJetMin is " << config->ptJetMin << std::endl;
  std::cout << "ptMuonMin is " << config->ptMuonMin << std::endl;
  std::cout << "jetAlg is " << config->jetAlg << std::endl;
  std::cout << "jetAlgR is " << config->jetAlgR << std::endl;

  TFile* f = new TFile("tree.root","RECREATE");
  TTree * tree = new TTree("tree","tree");

  TClonesArray * particles = new TClonesArray("TParticle",1000);
  TClonesArray * electrons = new TClonesArray("TParticle",10);
  TClonesArray * muons = new TClonesArray("TParticle",10);
  TClonesArray * photons = new TClonesArray("TParticle",10);
  TClonesArray * jets = new TClonesArray("TParticle",10);
  double met=0;
  double metPhi=0;
  tree->Branch("genParticles",particles);
  tree->Branch("electrons",electrons);
  tree->Branch("muons",muons);
  tree->Branch("photons",photons);
  tree->Branch("jets",jets);
  tree->Branch("met",&met,"met/F");
  tree->Branch("metPhi",&metPhi,"metPhi/F");

  HepMCImporter myImporter("test.hepmc");
  PhysicsObjectBuilder myBuilder(config);
  bool moreEvents=true;
  while(moreEvents)
  {
	myImporter.GenerateEvent();
	int nParts = myImporter.ImportParticles(particles);
	moreEvents = myImporter.Done();

	myBuilder.LoadGenParticles(particles);
	myBuilder.MakeElectrons(electrons);
	myBuilder.MakeMuons(muons);
	myBuilder.MakePhotons(photons);
	myBuilder.MakeJets(jets);

	tree->Fill();
  }

  f->cd();
  tree->Write();
  /*
  std::ifstream fin("test.yaml");
  YAML::Parser parser(fin);

  YAML::Node doc;
  while (parser.GetNextDocument(doc))
  {
    std::cout << "Begin Document" << std::endl;
    for(YAML::Iterator it = doc.begin(); it != doc.end(); ++it)
    {
	std::string key, value;
	it.first() >> key;
	it.second() >> value;
	std::cout << key << "\t" << value << std::endl;
    }
    std::cout << "End Document" << std::endl;
    if (const YAML::Node *value = doc.FindValue("name"))
    {
	std::string name;
	*value >> name;
	std::cout << "name is " << name << std::endl;
    }
    else
    {
	std::cout << "name doesn't exist\n";
    }

  }
  */

  return 0;
}
